@@ -1,7 +1,8 @@
-package Pcore::Chrome v0.12.0;
+package Pcore::Chrome v0.13.0;
 
 use Pcore -dist, -class, -res, -const;
 use Pcore::Chrome::Tab;
+use Pcore::Util::Scalar qw[weaken];
 use Pcore::Util::Data qw[from_json];
 
 has bin           => ();
@@ -10,7 +11,9 @@ has port          => 9222;
 has user_data_dir => sub { P->file1->tempdir };
 has useragent     => ();
 
-has _proc => ();
+has _proc     => ();
+has _pac      => ();
+has _pac_func => ();
 
 # https://chromedevtools.github.io/devtools-protocol/tot/
 # https://peter.sh/experiments/chromium-command-line-switches/
@@ -37,6 +40,10 @@ around new => sub ( $orig, $self, %args ) {
 
     $self->{port} ||= ( P->net->get_free_port( $args{host} ) or die q[Error get free port] );
 
+    $self->_build_pac_func( $args{proxy} );
+
+    $self->_run_pac;
+
     my $cmd = [
         qq["$args{bin}"],
 
@@ -63,6 +70,8 @@ around new => sub ( $orig, $self, %args ) {
         '--disable-web-security',
         '--allow-running-insecure-content',
 
+        qq[--proxy-pac-url="http://$self->{_pac}->{listen}->{host_port}/"],
+
         # logging
         # '--disable-logging',
         # '--log-level=0',
@@ -126,6 +135,70 @@ sub new_tab ( $self, $url = undef ) {
     return $tab;
 }
 
+sub reload_pac ($self) {
+    my $tab = $self->new_tab('about:blank');
+
+    my $res = $tab->navigate_to('chrome://net-internals/#proxy');
+
+    $tab->('Runtime.enable');
+
+    $res = $tab->( 'Runtime.evaluate', { expression => q[document.getElementById("proxy-view-reload-settings").click()] } );
+
+    return;
+}
+
+sub set_proxy ( $self, $proxy ) {
+    $self->_build_pac_func($proxy);
+
+    $self->reload_pac;
+
+    return;
+}
+
+sub _build_pac_func ( $self, $proxy ) {
+    my $pac_proxy;
+
+    if ( !$proxy ) {
+        $pac_proxy = 'DIRECT';
+    }
+    else {
+        my $uri = P->uri($proxy);
+
+        if ( !$uri ) {
+            $pac_proxy = 'DIRECT';
+        }
+        elsif ( $uri->{scheme} eq 'socks' ) {
+            $pac_proxy = "SOCKS $uri->{host_port}";
+        }
+        else {
+            $pac_proxy = "PROXY $uri->{host_port}";
+        }
+    }
+
+    $self->{_pac_func} = <<"JS";
+function FindProxyForURL ( url, host ) {
+    return "$pac_proxy";
+}
+JS
+
+    return;
+}
+
+sub _run_pac ($self) {
+    require Pcore::HTTP::Server;
+
+    weaken $self;
+
+    $self->{_pac} = Pcore::HTTP::Server->new(
+        listen     => '//127.0.0.1:9999',
+        on_request => sub ($req) {
+            return 200, [ 'Content-Type' => 'application/x-ns-proxy-autoconfig' ], $self->{_pac_func} || $EMPTY;
+        }
+    );
+
+    return;
+}
+
 1;
 ## -----SOURCE FILTER LOG BEGIN-----
 ##
@@ -133,9 +206,9 @@ sub new_tab ( $self, $url = undef ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    2 | 80                   | CodeLayout::ProhibitQuotedWordLists - List of quoted literal words                                             |
+## |    2 | 89                   | CodeLayout::ProhibitQuotedWordLists - List of quoted literal words                                             |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    1 | 92                   | CodeLayout::ProhibitParensWithBuiltins - Builtin function called with parentheses                              |
+## |    1 | 101                  | CodeLayout::ProhibitParensWithBuiltins - Builtin function called with parentheses                              |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
